@@ -29,6 +29,10 @@ try {
   var Decipher = binding.Decipher;
   var Sign = binding.Sign;
   var Verify = binding.Verify;
+  var DiffieHellman = binding.DiffieHellman;
+  var PBKDF2 = binding.PBKDF2;
+  var randomBytes = binding.randomBytes;
+  var pseudoRandomBytes = binding.pseudoRandomBytes;
   var crypto = true;
 } catch (e) {
 
@@ -36,7 +40,7 @@ try {
 }
 
 
-function Credentials(secureProtocol, context) {
+function Credentials(secureProtocol, flags, context) {
   if (!(this instanceof Credentials)) {
     return new Credentials(secureProtocol);
   }
@@ -56,6 +60,8 @@ function Credentials(secureProtocol, context) {
       this.context.init();
     }
   }
+
+  if (flags) this.context.setOptions(flags);
 }
 
 exports.Credentials = Credentials;
@@ -63,14 +69,25 @@ exports.Credentials = Credentials;
 
 exports.createCredentials = function(options, context) {
   if (!options) options = {};
-  var c = new Credentials(options.secureProtocol, context);
+
+  var c = new Credentials(options.secureProtocol,
+                          options.secureOptions,
+                          context);
 
   if (context) return c;
 
-  if (options.key) c.context.setKey(options.key);
+  if (options.key) {
+    if (options.passphrase) {
+      c.context.setKey(options.key, options.passphrase);
+    } else {
+      c.context.setKey(options.key);
+    }
+  }
 
   if (options.cert) c.context.setCert(options.cert);
 
+  if (options.ciphers) c.context.setCiphers(options.ciphers);
+
   if (options.ca) {
     if (Array.isArray(options.ca)) {
       for (var i = 0, len = options.ca.length; i < len; i++) {
@@ -85,7 +102,7 @@ exports.createCredentials = function(options, context) {
 
   if (options.crl) {
     if (Array.isArray(options.crl)) {
-      for(var i = 0, len = options.crl.length; i < len; i++) {
+      for (var i = 0, len = options.crl.length; i < len; i++) {
         c.context.addCRL(options.crl[i]);
       }
     } else {
@@ -93,6 +110,18 @@ exports.createCredentials = function(options, context) {
     }
   }
 
+  if (options.sessionIdContext) {
+    c.context.setSessionIdContext(options.sessionIdContext);
+  }
+
+  if (options.pfx) {
+    if (options.passphrase) {
+      c.context.loadPKCS12(options.pfx, options.passphrase);
+    } else {
+      c.context.loadPKCS12(options.pfx);
+    }
+  }
+
   return c;
 };
 
@@ -140,3 +169,23 @@ exports.Verify = Verify;
 exports.createVerify = function(algorithm) {
   return (new Verify).init(algorithm);
 };
+
+exports.DiffieHellman = DiffieHellman;
+exports.createDiffieHellman = function(size_or_key, enc) {
+  if (!size_or_key) {
+    return new DiffieHellman();
+  } else if (!enc) {
+    return new DiffieHellman(size_or_key);
+  } else {
+    return new DiffieHellman(size_or_key, enc);
+  }
+
+};
+
+exports.pbkdf2 = PBKDF2;
+
+exports.randomBytes = randomBytes;
+exports.pseudoRandomBytes = pseudoRandomBytes;
+
+exports.rng = randomBytes;
+exports.prng = pseudoRandomBytes;
