@@ -19,9 +19,11 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 // USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-var EventEmitter = exports.EventEmitter = process.EventEmitter;
 var isArray = Array.isArray;
 
+function EventEmitter() { }
+exports.EventEmitter = EventEmitter;
+
 // By default EventEmitters will print a warning if more than
 // 10 listeners are added to it. This is a useful default which
 // helps finding memory leaks.
@@ -31,11 +33,12 @@ var isArray = Array.isArray;
 var defaultMaxListeners = 10;
 EventEmitter.prototype.setMaxListeners = function(n) {
   if (!this._events) this._events = {};
-  this._events.maxListeners = n;
+  this._maxListeners = n;
 };
 
 
-EventEmitter.prototype.emit = function(type) {
+EventEmitter.prototype.emit = function() {
+  var type = arguments[0];
   // If there is no 'error' event listener then throw.
   if (type === 'error') {
     if (!this._events || !this._events.error ||
@@ -68,13 +71,17 @@ EventEmitter.prototype.emit = function(type) {
         break;
       // slower
       default:
-        var args = Array.prototype.slice.call(arguments, 1);
+        var l = arguments.length;
+        var args = new Array(l - 1);
+        for (var i = 1; i < l; i++) args[i - 1] = arguments[i];
         handler.apply(this, args);
     }
     return true;
 
   } else if (isArray(handler)) {
-    var args = Array.prototype.slice.call(arguments, 1);
+    var l = arguments.length;
+    var args = new Array(l - 1);
+    for (var i = 1; i < l; i++) args[i - 1] = arguments[i];
 
     var listeners = handler.slice();
     for (var i = 0, l = listeners.length; i < l; i++) {
@@ -108,27 +115,29 @@ EventEmitter.prototype.addListener = function(type, listener) {
     // If we've already got an array, just append.
     this._events[type].push(listener);
 
-    // Check for listener leak
-    if (!this._events[type].warned) {
-      var m;
-      if (this._events.maxListeners !== undefined) {
-        m = this._events.maxListeners;
-      } else {
-        m = defaultMaxListeners;
-      }
-
-      if (m && m > 0 && this._events[type].length > m) {
-        this._events[type].warned = true;
-        console.error('(node) warning: possible EventEmitter memory ' +
-                      'leak detected. %d listeners added. ' +
-                      'Use emitter.setMaxListeners() to increase limit.',
-                      this._events[type].length);
-        console.trace();
-      }
-    }
   } else {
     // Adding the second element, need to change to array.
     this._events[type] = [this._events[type], listener];
+
+  }
+
+  // Check for listener leak
+  if (isArray(this._events[type]) && !this._events[type].warned) {
+    var m;
+    if (this._maxListeners !== undefined) {
+      m = this._maxListeners;
+    } else {
+      m = defaultMaxListeners;
+    }
+
+    if (m && m > 0 && this._events[type].length > m) {
+      this._events[type].warned = true;
+      console.error('(node) warning: possible EventEmitter memory ' +
+                    'leak detected. %d listeners added. ' +
+                    'Use emitter.setMaxListeners() to increase limit.',
+                    this._events[type].length);
+      console.trace();
+    }
   }
 
   return this;
@@ -137,6 +146,10 @@ EventEmitter.prototype.addListener = function(type, listener) {
 EventEmitter.prototype.on = EventEmitter.prototype.addListener;
 
 EventEmitter.prototype.once = function(type, listener) {
+  if ('function' !== typeof listener) {
+    throw new Error('.once only takes instances of Function');
+  }
+
   var self = this;
   function g() {
     self.removeListener(type, g);
@@ -184,6 +197,11 @@ EventEmitter.prototype.removeListener = function(type, listener) {
 };
 
 EventEmitter.prototype.removeAllListeners = function(type) {
+  if (arguments.length === 0) {
+    this._events = {};
+    return this;
+  }
+
   // does not use listeners(), so no side effect of creating _events[type]
   if (type && this._events && this._events[type]) this._events[type] = null;
   return this;
