@@ -78,6 +78,7 @@ fs.readFile = function(path, encoding_) {
   var readStream = fs.createReadStream(path);
   var buffers = [];
   var nread = 0;
+  var error;
 
   readStream.on('data', function(chunk) {
     buffers.push(chunk);
@@ -85,11 +86,18 @@ fs.readFile = function(path, encoding_) {
   });
 
   readStream.on('error', function(er) {
-    callback(er);
+    error = er;
     readStream.destroy();
+    if (!readStream.fd) {
+      readStream.emit('close');
+    }
   });
 
-  readStream.on('end', function() {
+  readStream.on('close', function() {
+    if (error) {
+      return callback(error);
+    }
+
     // copy all the buffers into one
     var buffer;
     switch (buffers.length) {
@@ -123,17 +131,19 @@ fs.readFileSync = function(path, encoding) {
   var nread = 0;
   var lastRead = 0;
 
-  do {
-    if (lastRead) {
-      buffer._bytesRead = lastRead;
-      nread += lastRead;
-      buffers.push(buffer);
-    }
-    var buffer = new Buffer(4048);
-    lastRead = fs.readSync(fd, buffer, 0, buffer.length, null);
-  } while (lastRead > 0);
-
-  fs.closeSync(fd);
+  try {
+    do {
+      if (lastRead) {
+        buffer._bytesRead = lastRead;
+        nread += lastRead;
+        buffers.push(buffer);
+      }
+      var buffer = new Buffer(4048);
+      lastRead = fs.readSync(fd, buffer, 0, buffer.length, null);
+    } while (lastRead > 0);
+  } finally {
+    fs.closeSync(fd);
+  }
 
   if (buffers.length > 1) {
     var offset = 0;
@@ -621,11 +631,13 @@ fs.writeFileSync = function(path, data, encoding) {
   }
   var written = 0;
   var length = data.length;
-  //writeSync(fd, buffer, offset, length, position)
-  while (written < length) {
-    written += fs.writeSync(fd, data, written, length - written, written);
+  try {
+    while (written < length) {
+      written += fs.writeSync(fd, data, written, length - written, written);
+    }
+  } finally {
+    fs.closeSync(fd);
   }
-  fs.closeSync(fd);
 };
 
 
@@ -647,6 +659,7 @@ function FSWatcher() {
 
   this._handle.onchange = function(status, event, filename) {
     if (status) {
+      self._handle.close();
       self.emit('error', errnoException(errno, 'watch'));
     } else {
       self.emit('change', event, filename);
@@ -1055,7 +1068,9 @@ var ReadStream = fs.ReadStream = function(path, options) {
   }
 
   if (this.fd !== null) {
-    this._read();
+    process.nextTick(function () {
+      self._read();
+    });
     return;
   }
 
@@ -1159,6 +1174,8 @@ ReadStream.prototype._emitData = function(d) {
 
 ReadStream.prototype.destroy = function(cb) {
   var self = this;
+
+  if (!this.readable) return;
   this.readable = false;
 
   function close() {
@@ -1174,10 +1191,10 @@ ReadStream.prototype.destroy = function(cb) {
     });
   }
 
-  if (this.fd) {
-    close();
-  } else {
+  if (this.fd === null) {
     this.addListener('open', close);
+  } else {
+    close();
   }
 };
 
@@ -1191,8 +1208,9 @@ ReadStream.prototype.resume = function() {
   this.paused = false;
 
   if (this.buffer) {
-    this._emitData(this.buffer);
+    var buffer = this.buffer;
     this.buffer = null;
+    this._emitData(buffer);
   }
 
   // hasn't opened yet.
@@ -1245,7 +1263,8 @@ var WriteStream = fs.WriteStream = function(path, options) {
   this._queue = [];
 
   if (this.fd === null) {
-    this._queue.push([fs.open, this.path, this.flags, this.mode, undefined]);
+    this._open = fs.open;
+    this._queue.push([this._open, this.path, this.flags, this.mode, undefined]);
     this.flush();
   }
 };
@@ -1287,7 +1306,7 @@ WriteStream.prototype.flush = function() {
         cb(null, arguments[1]);
       }
 
-    } else if (method === fs.open) {
+    } else if (method === self._open) {
       // save reference for file pointer
       self.fd = arguments[1];
       self.emit('open', self.fd);
@@ -1305,7 +1324,7 @@ WriteStream.prototype.flush = function() {
   });
 
   // Inject the file pointer
-  if (method !== fs.open) {
+  if (method !== self._open) {
     args.unshift(this.fd);
   }
 
@@ -1358,6 +1377,8 @@ WriteStream.prototype.end = function(data, encoding, cb) {
 
 WriteStream.prototype.destroy = function(cb) {
   var self = this;
+
+  if (!this.writable) return;
   this.writable = false;
 
   function close() {
@@ -1373,10 +1394,10 @@ WriteStream.prototype.destroy = function(cb) {
     });
   }
 
-  if (this.fd) {
-    close();
-  } else {
+  if (this.fd === null) {
     this.addListener('open', close);
+  } else {
+    close();
   }
 };
 
