@@ -35,6 +35,108 @@ if (process.env.NODE_DEBUG && /http/.test(process.env.NODE_DEBUG)) {
   debug = function() { };
 }
 
+function parserOnHeaders(headers, url) {
+  this._headers = this._headers.concat(headers);
+  this._url += url;
+}
+
+// info.headers and info.url are set only if .onHeaders()
+// has not been called for this request.
+//
+// info.url is not set for response parsers but that's not
+// applicable here since all our parsers are request parsers.
+function parserOnHeadersComplete(info) {
+  var parser = this;
+  var headers = info.headers;
+  var url = info.url;
+
+  if (!headers) {
+    headers = parser._headers;
+    parser._headers = [];
+  }
+
+  if (!url) {
+    url = parser._url;
+    parser._url = '';
+  }
+
+  parser.incoming = new IncomingMessage(parser.socket);
+  parser.incoming.httpVersionMajor = info.versionMajor;
+  parser.incoming.httpVersionMinor = info.versionMinor;
+  parser.incoming.httpVersion = info.versionMajor + '.' + info.versionMinor;
+  parser.incoming.url = url;
+
+  for (var i = 0, n = headers.length; i < n; i += 2) {
+    var k = headers[i];
+    var v = headers[i + 1];
+    parser.incoming._addHeaderLine(k.toLowerCase(), v);
+  }
+
+  if (info.method) {
+    // server only
+    parser.incoming.method = info.method;
+  } else {
+    // client only
+    parser.incoming.statusCode = info.statusCode;
+    // CHECKME dead code? we're always a request parser
+  }
+
+  parser.incoming.upgrade = info.upgrade;
+
+  var isHeadResponse = false;
+
+  if (!info.upgrade) {
+    // For upgraded connections, we'll emit this after parser.execute
+    // so that we can capture the first part of the new protocol
+    isHeadResponse = parser.onIncoming(parser.incoming, info.shouldKeepAlive);
+  }
+
+  return isHeadResponse;
+}
+
+function parserOnBody(b, start, len) {
+  var parser = this;
+  // TODO body encoding?
+  var slice = b.slice(start, start + len);
+  if (parser.incoming._decoder) {
+    var string = parser.incoming._decoder.write(slice);
+    if (string.length) parser.incoming.emit('data', string);
+  } else {
+    parser.incoming.emit('data', slice);
+  }
+}
+
+function parserOnMessageComplete() {
+  var parser = this;
+  parser.incoming.complete = true;
+
+  // Emit any trailing headers.
+  var headers = parser._headers;
+  if (headers) {
+    for (var i = 0, n = headers.length; i < n; i += 2) {
+      var k = headers[i];
+      var v = headers[i + 1];
+      parser.incoming._addHeaderLine(k.toLowerCase(), v);
+    }
+    parser._headers = [];
+    parser._url = '';
+  }
+
+  if (!parser.incoming.upgrade) {
+    // For upgraded connections, also emit this after parser.execute
+    parser.incoming.readable = false;
+    if (!parser.incoming._ended) {
+      parser.incoming._ended = true;
+      parser.incoming.emit('end');
+    }
+  }
+
+  if (parser.socket.readable) {
+    // force to read the next incoming message
+    parser.socket.resume();
+  }
+}
+
 
 var parsers = new FreeList('parsers', 1000, function() {
   var parser = new HTTPParser(HTTPParser.REQUEST);
@@ -47,101 +149,10 @@ var parsers = new FreeList('parsers', 1000, function() {
   // across multiple TCP packets or too large to be
   // processed in a single run. This method is also
   // called to process trailing HTTP headers.
-  parser.onHeaders = function(headers, url) {
-    parser._headers = parser._headers.concat(headers);
-    parser._url += url;
-  };
-
-  // info.headers and info.url are set only if .onHeaders()
-  // has not been called for this request.
-  //
-  // info.url is not set for response parsers but that's not
-  // applicable here since all our parsers are request parsers.
-  parser.onHeadersComplete = function(info) {
-    var headers = info.headers;
-    var url = info.url;
-
-    if (!headers) {
-      headers = parser._headers;
-      parser._headers = [];
-    }
-
-    if (!url) {
-      url = parser._url;
-      parser._url = '';
-    }
-
-    parser.incoming = new IncomingMessage(parser.socket);
-    parser.incoming.httpVersionMajor = info.versionMajor;
-    parser.incoming.httpVersionMinor = info.versionMinor;
-    parser.incoming.httpVersion = info.versionMajor + '.' + info.versionMinor;
-    parser.incoming.url = url;
-
-    for (var i = 0, n = headers.length; i < n; i += 2) {
-      var k = headers[i];
-      var v = headers[i + 1];
-      parser.incoming._addHeaderLine(k.toLowerCase(), v);
-    }
-
-    if (info.method) {
-      // server only
-      parser.incoming.method = info.method;
-    } else {
-      // client only
-      parser.incoming.statusCode = info.statusCode;
-      // CHECKME dead code? we're always a request parser
-    }
-
-    parser.incoming.upgrade = info.upgrade;
-
-    var isHeadResponse = false;
-
-    if (!info.upgrade) {
-      // For upgraded connections, we'll emit this after parser.execute
-      // so that we can capture the first part of the new protocol
-      isHeadResponse = parser.onIncoming(parser.incoming, info.shouldKeepAlive);
-    }
-
-    return isHeadResponse;
-  };
-
-  parser.onBody = function(b, start, len) {
-    // TODO body encoding?
-    var slice = b.slice(start, start + len);
-    if (parser.incoming._decoder) {
-      var string = parser.incoming._decoder.write(slice);
-      if (string.length) parser.incoming.emit('data', string);
-    } else {
-      parser.incoming.emit('data', slice);
-    }
-  };
-
-  parser.onMessageComplete = function() {
-    parser.incoming.complete = true;
-
-    // Emit any trailing headers.
-    var headers = parser._headers;
-    if (headers) {
-      for (var i = 0, n = headers.length; i < n; i += 2) {
-        var k = headers[i];
-        var v = headers[i + 1];
-        parser.incoming._addHeaderLine(k.toLowerCase(), v);
-      }
-      parser._headers = [];
-      parser._url = '';
-    }
-
-    if (!parser.incoming.upgrade) {
-      // For upgraded connections, also emit this after parser.execute
-      parser.incoming.readable = false;
-      parser.incoming.emit('end');
-    }
-
-    if (parser.socket.readable) {
-      // force to read the next incoming message
-      parser.socket.resume();
-    }
-  };
+  parser.onHeaders = parserOnHeaders;
+  parser.onHeadersComplete = parserOnHeadersComplete;
+  parser.onBody = parserOnBody;
+  parser.onMessageComplete = parserOnMessageComplete;
 
   return parser;
 });
@@ -483,7 +494,9 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
   // keep-alive logic
   if (sentConnectionHeader === false) {
     if (this.shouldKeepAlive &&
-        (sentContentLengthHeader || this.useChunkedEncodingByDefault || this.agent)) {
+        (sentContentLengthHeader ||
+         this.useChunkedEncodingByDefault ||
+         this.agent)) {
       messageHeader += 'Connection: keep-alive\r\n';
     } else {
       this._last = true;
@@ -516,7 +529,7 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
 
 OutgoingMessage.prototype.setHeader = function(name, value) {
   if (arguments.length < 2) {
-    throw new Error("`name` and `value` are required for setHeader().");
+    throw new Error('`name` and `value` are required for setHeader().');
   }
 
   if (this._header) {
@@ -533,7 +546,7 @@ OutgoingMessage.prototype.setHeader = function(name, value) {
 
 OutgoingMessage.prototype.getHeader = function(name) {
   if (arguments.length < 1) {
-    throw new Error("`name` is required for getHeader().");
+    throw new Error('`name` is required for getHeader().');
   }
 
   if (!this._headers) return;
@@ -545,7 +558,7 @@ OutgoingMessage.prototype.getHeader = function(name) {
 
 OutgoingMessage.prototype.removeHeader = function(name) {
   if (arguments.length < 1) {
-    throw new Error("`name` is required for removeHeader().");
+    throw new Error('`name` is required for removeHeader().');
   }
 
   if (this._header) {
@@ -1014,7 +1027,7 @@ function ClientRequest(options, cb) {
   var method = self.method = (options.method || 'GET').toUpperCase();
   self.path = options.path || '/';
   if (cb) {
-    self.on('response', cb);
+    self.once('response', cb);
   }
 
   if (!Array.isArray(options.headers)) {
@@ -1071,7 +1084,9 @@ function ClientRequest(options, cb) {
     self._last = true;
     self.shouldKeepAlive = false;
     if (options.createConnection) {
-      self.onSocket(options.createConnection(options.port, options.host, options));
+      self.onSocket(options.createConnection(options.port,
+                                             options.host,
+                                             options));
     } else {
       self.onSocket(net.createConnection(options.port, options.host));
     }
@@ -1079,6 +1094,7 @@ function ClientRequest(options, cb) {
 
   self._deferToConnect(null, null, function() {
     self._flush();
+    self = null;
   });
 
 }
@@ -1087,7 +1103,8 @@ util.inherits(ClientRequest, OutgoingMessage);
 exports.ClientRequest = ClientRequest;
 
 ClientRequest.prototype._implicitHeader = function() {
-  this._storeHeader(this.method + ' ' + this.path + ' HTTP/1.1\r\n', this._renderHeaders());
+  this._storeHeader(this.method + ' ' + this.path + ' HTTP/1.1\r\n',
+                    this._renderHeaders());
 };
 
 ClientRequest.prototype.abort = function() {
@@ -1109,171 +1126,227 @@ function createHangUpError() {
   return error;
 }
 
+// Free the parser and also break any links that it
+// might have to any other things.
+// TODO: All parser data should be attached to a
+// single object, so that it can be easily cleaned
+// up by doing `parser.data = {}`, which should
+// be done in FreeList.free.  `parsers.free(parser)`
+// should be all that is needed.
+function freeParser(parser, req) {
+  if (parser) {
+    parser._headers = [];
+    parser.onIncoming = null;
+    if (parser.socket) {
+      parser.socket.onend = null;
+      parser.socket.ondata = null;
+      parser.socket.parser = null;
+    }
+    parser.socket = null;
+    parser.incoming = null;
+    parsers.free(parser);
+    parser = null;
+  }
+  if (req) {
+    req.parser = null;
+  }
+}
+
+function socketCloseListener() {
+  var socket = this;
+  var parser = socket.parser;
+  var req = socket._httpMessage;
+  debug('HTTP socket close');
+  req.emit('close');
+  if (req.res && req.res.readable) {
+    // Socket closed before we emitted "end" below.
+    req.res.emit('aborted');
+    if (!req.res._ended) {
+      req.res._ended = true;
+      req.res.emit('end');
+    }
+    req.res.emit('close');
+  } else if (!req.res && !req._hadError) {
+    // This socket error fired before we started to
+    // receive a response. The error needs to
+    // fire on the request.
+    req.emit('error', createHangUpError());
+  }
+
+  if (parser) {
+    parser.finish();
+    freeParser(parser, req);
+  }
+}
+
+function socketErrorListener(err) {
+  var socket = this;
+  var parser = socket.parser;
+  var req = socket._httpMessage;
+  debug('HTTP SOCKET ERROR: ' + err.message + '\n' + err.stack);
+  if (req) {
+    req.emit('error', err);
+    // For Safety. Some additional errors might fire later on
+    // and we need to make sure we don't double-fire the error event.
+    req._hadError = true;
+  }
+  if (parser) {
+    parser.finish();
+    freeParser(parser, req);
+  }
+  socket.destroy();
+}
+
+function responseOnEnd() {
+  var req = this.req;
+  var socket = req.socket;
+
+  if (req.shouldKeepAlive) {
+    debug('AGENT socket keep-alive');
+    socket.removeListener('close', socketCloseListener);
+    socket.removeListener('error', socketErrorListener);
+    socket.emit('free');
+  } else {
+    if (socket.writable) {
+      debug('AGENT socket.destroySoon()');
+      socket.destroySoon();
+    }
+    assert(!socket.writable);
+  }
+}
+
+function parserOnIncomingClient(res, shouldKeepAlive) {
+  var parser = this;
+  var socket = this.socket;
+  var req = socket._httpMessage;
+
+  debug('AGENT incoming response!');
+
+  if (req.res) {
+    // We already have a response object, this means the server
+    // sent a double response.
+    socket.destroy();
+    return;
+  }
+  req.res = res;
+
+  // Responses to HEAD requests are crazy.
+  // HEAD responses aren't allowed to have an entity-body
+  // but *can* have a content-length which actually corresponds
+  // to the content-length of the entity-body had the request
+  // been a GET.
+  var isHeadResponse = req.method == 'HEAD';
+  debug('AGENT isHeadResponse ' + isHeadResponse);
+
+  if (res.statusCode == 100) {
+    // restart the parser, as this is a continue message.
+    delete req.res; // Clear res so that we don't hit double-responses.
+    req.emit('continue');
+    return true;
+  }
+
+  if (req.shouldKeepAlive && !shouldKeepAlive && !req.upgraded) {
+    // Server MUST respond with Connection:keep-alive for us to enable it.
+    // If we've been upgraded (via WebSockets) we also shouldn't try to
+    // keep the connection open.
+    req.shouldKeepAlive = false;
+  }
+
+  DTRACE_HTTP_CLIENT_RESPONSE(socket, req);
+  req.emit('response', res);
+  req.res = res;
+  res.req = req;
+
+  res.on('end', responseOnEnd);
+
+  return isHeadResponse;
+}
+
+function socketOnEnd() {
+  var socket = this;
+  var req = this._httpMessage;
+  var parser = this.parser;
+  if (!req.res) {
+    // If we don't have a response then we know that the socket
+    // ended prematurely and we need to emit an error on the request.
+    req.emit('error', createHangUpError());
+    req._hadError = true;
+  }
+  if (parser) {
+    parser.finish();
+    freeParser(parser, req);
+  }
+  socket.destroy();
+}
+
+
+function socketOnData(d, start, end) {
+  var socket = this;
+  var req = this._httpMessage;
+  var parser = this.parser;
+
+  var ret = parser.execute(d, start, end - start);
+  if (ret instanceof Error) {
+    debug('parse error');
+    freeParser(parser, req);
+    socket.destroy(ret);
+  } else if (parser.incoming && parser.incoming.upgrade) {
+    var bytesParsed = ret;
+    socket.ondata = null;
+    socket.onend = null;
+
+    var res = parser.incoming;
+    req.res = res;
+
+    // This is start + byteParsed
+    var upgradeHead = d.slice(start + bytesParsed, end);
+    if (req.listeners('upgrade').length) {
+      // Emit 'upgrade' on the Agent.
+      req.upgraded = true;
+      req.emit('upgrade', res, socket, upgradeHead);
+      socket.emit('agentRemove');
+    } else {
+      // Got upgrade header, but have no handler.
+      socket.destroy();
+    }
+    freeParser(parser, req);
+  } else if (parser.incoming && parser.incoming.complete &&
+             // When the status code is 100 (Continue), the server will
+             // send a final response after this client sends a request
+             // body. So, we must not free the parser.
+             parser.incoming.statusCode !== 100) {
+    freeParser(parser, req);
+  }
+}
 
 ClientRequest.prototype.onSocket = function(socket) {
   var req = this;
+
   process.nextTick(function() {
     var parser = parsers.alloc();
+
     req.socket = socket;
     req.connection = socket;
-    parser.reinitialize(HTTPParser.RESPONSE);
     parser.socket = socket;
+    socket.parser = parser;
+    parser.reinitialize(HTTPParser.RESPONSE);
     parser.incoming = null;
     req.parser = parser;
 
     socket._httpMessage = req;
+
     // Setup "drain" propogation.
     httpSocketSetup(socket);
+    socket.ondata = socketOnData;
+    socket.onend = socketOnEnd;
+    socket.on('error', socketErrorListener);
+    socket.on('close', socketCloseListener);
+    parser.onIncoming = parserOnIncomingClient;
 
-    var freeParser = function() {
-      if (parser) {
-        parsers.free(parser);
-        parser = null;
-      }
-    };
-
-    var errorListener = function(err) {
-      debug('HTTP SOCKET ERROR: ' + err.message + '\n' + err.stack);
-      req.emit('error', err);
-      // For Safety. Some additional errors might fire later on
-      // and we need to make sure we don't double-fire the error event.
-      req._hadError = true;
-      if (parser) {
-        parser.finish();
-        freeParser();
-      }
-      socket.destroy();
-    }
-    socket.on('error', errorListener);
-
-    socket.ondata = function(d, start, end) {
-      var ret = parser.execute(d, start, end - start);
-      if (ret instanceof Error) {
-        debug('parse error');
-        freeParser();
-        socket.destroy(ret);
-      } else if (parser.incoming && parser.incoming.upgrade) {
-        var bytesParsed = ret;
-        socket.ondata = null;
-        socket.onend = null;
-
-        var res = parser.incoming;
-        req.res = res;
-
-        // This is start + byteParsed
-        var upgradeHead = d.slice(start + bytesParsed, end);
-        if (req.listeners('upgrade').length) {
-          // Emit 'upgrade' on the Agent.
-          req.upgraded = true;
-          req.emit('upgrade', res, socket, upgradeHead);
-          socket.emit('agentRemove');
-        } else {
-          // Got upgrade header, but have no handler.
-          socket.destroy();
-        }
-        freeParser();
-      } else if (parser.incoming && parser.incoming.complete &&
-                 // When the status code is 100 (Continue), the server will
-                 // send a final response after this client sends a request
-                 // body. So, we must not free the parser.
-                 parser.incoming.statusCode !== 100) {
-        freeParser();
-      }
-    };
-
-    socket.onend = function() {
-      if (!req.res) {
-        // If we don't have a response then we know that the socket
-        // ended prematurely and we need to emit an error on the request.
-        req.emit('error', createHangUpError());
-        req._hadError = true;
-      }
-      if (parser) {
-        parser.finish();
-        freeParser();
-      }
-      socket.destroy();
-    };
-
-    var closeListener = function() {
-      debug('HTTP socket close');
-      req.emit('close');
-      if (req.res && req.res.readable) {
-        // Socket closed before we emitted "end" below.
-        req.res.emit('aborted');
-        req.res.emit('end');
-        req.res.emit('close');
-      } else if (!req.res && !req._hadError) {
-        // This socket error fired before we started to
-        // receive a response. The error needs to
-        // fire on the request.
-        req.emit('error', createHangUpError());
-      }
-    }
-    socket.on('close', closeListener);
-
-    parser.onIncoming = function(res, shouldKeepAlive) {
-      debug('AGENT incoming response!');
-
-      if (req.res) {
-        // We already have a response object, this means the server
-        // sent a double response.
-        socket.destroy();
-        return;
-      }
-      req.res = res;
-
-      // Responses to HEAD requests are crazy.
-      // HEAD responses aren't allowed to have an entity-body
-      // but *can* have a content-length which actually corresponds
-      // to the content-length of the entity-body had the request
-      // been a GET.
-      var isHeadResponse = req.method == 'HEAD';
-      debug('AGENT isHeadResponse ' + isHeadResponse);
-
-      if (res.statusCode == 100) {
-        // restart the parser, as this is a continue message.
-        delete req.res; // Clear res so that we don't hit double-responses.
-        req.emit('continue');
-        return true;
-      }
-
-      if (req.shouldKeepAlive && !shouldKeepAlive && !req.upgraded) {
-        // Server MUST respond with Connection:keep-alive for us to enable it.
-        // If we've been upgraded (via WebSockets) we also shouldn't try to
-        // keep the connection open.
-        req.shouldKeepAlive = false;
-      }
-
-      res.addListener('end', function() {
-        if (!req.shouldKeepAlive) {
-          if (socket.writable) {
-            debug('AGENT socket.destroySoon()');
-            socket.destroySoon();
-          }
-          assert(!socket.writable);
-        } else {
-          debug('AGENT socket keep-alive');
-        }
-      });
-
-      DTRACE_HTTP_CLIENT_RESPONSE(socket, req);
-      req.emit('response', res);
-
-      res.on('end', function() {
-        if (req.shouldKeepAlive) {
-          socket.removeListener('close', closeListener);
-          socket.removeListener('error', errorListener);
-          socket.emit('free');
-        }
-      });
-
-      return isHeadResponse;
-    };
     req.emit('socket', socket);
   });
-
 };
+
 ClientRequest.prototype._deferToConnect = function(method, arguments_, cb) {
   // This function is for calls that need to happen once the socket is
   // connected and writable. It's an important promisy thing for all the socket
@@ -1302,9 +1375,32 @@ ClientRequest.prototype._deferToConnect = function(method, arguments_, cb) {
     onSocket();
   }
 };
-ClientRequest.prototype.setTimeout = function() {
-  this._deferToConnect('setTimeout', arguments);
+
+ClientRequest.prototype.setTimeout = function(msecs, callback) {
+  if (callback) this.once('timeout', callback);
+
+  var self = this;
+  function emitTimeout() {
+    self.emit('timeout');
+  }
+
+  if (this.socket && this.socket.writable) {
+    this.socket.setTimeout(msecs, emitTimeout);
+    return;
+  }
+
+  if (this.socket) {
+    this.socket.once('connect', function() {
+      this.setTimeout(msecs, emitTimeout);
+    });
+    return;
+  }
+
+  this.once('socket', function(sock) {
+    this.setTimeout(msecs, emitTimeout);
+  });
 };
+
 ClientRequest.prototype.setNoDelay = function() {
   this._deferToConnect('setNoDelay', arguments);
 };
@@ -1393,7 +1489,7 @@ function connectionListener(socket) {
   httpSocketSetup(socket);
 
   socket.setTimeout(2 * 60 * 1000); // 2 minute timeout
-  socket.addListener('timeout', function() {
+  socket.once('timeout', function() {
     socket.destroy();
   });
 
@@ -1453,8 +1549,8 @@ function connectionListener(socket) {
 
   socket.addListener('close', function() {
     debug('server socket close');
-    // unref the parser for easy gc
-    parsers.free(parser);
+    // mark this parser as reusable
+    freeParser(parser);
 
     abortIncoming();
   });
