@@ -304,11 +304,16 @@ Module._load = function(request, parent, isMain) {
   }
 
   Module._cache[filename] = module;
+
+  var hadException = true;
+
   try {
     module.load(filename);
-  } catch (err) {
-    delete Module._cache[filename];
-    throw err;
+    hadException = false;
+  } finally {
+    if (hadException) {
+      delete Module._cache[filename];
+    }
   }
 
   return module.exports;
