@@ -177,6 +177,9 @@ if (isWindows) {
       tail += '\\';
     }
 
+    // Convert slashes to backslashes when `device` points to an UNC root.
+    device = device.replace(/\//g, '\\');
+
     return device + (isAbsolute ? '\\' : '') + tail;
   };
 
