@@ -20,7 +20,7 @@
 // USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 // Inspiration for this code comes from Salvatore Sanfilippo's linenoise.
-// http://github.com/antirez/linenoise
+// https://github.com/antirez/linenoise
 // Reference:
 // * http://invisible-island.net/xterm/ctlseqs/ctlseqs.html
 // * http://www.3waylabs.com/nw/WWW/products/wizcon/vt220.html
@@ -45,13 +45,22 @@ function Interface(input, output, completer) {
   }
   EventEmitter.call(this);
 
+  completer = completer || function() { return []; };
+
+  if (typeof completer !== 'function') {
+    throw new TypeError("Argument 'completer' must be a function");
+  }
+
   var self = this;
 
   this.output = output;
   this.input = input;
   input.resume();
 
-  this.completer = completer;
+  // Check arity, 2 - for async, 1 for sync
+  this.completer = completer.length === 2 ? completer : function(v, callback) {
+    callback(null, completer(v));
+  };
 
   this.setPrompt('> ');
 
@@ -86,13 +95,13 @@ function Interface(input, output, completer) {
     this.history = [];
     this.historyIndex = -1;
 
-    var winSize = tty.getWindowSize(output.fd);
-    exports.columns = winSize[1];
+    var winSize = output.getWindowSize();
+    exports.columns = winSize[0];
 
     if (process.listeners('SIGWINCH').length === 0) {
       process.on('SIGWINCH', function() {
-        var winSize = tty.getWindowSize(output.fd);
-        exports.columns = winSize[1];
+        var winSize = output.getWindowSize();
+        exports.columns = winSize[0];
       });
     }
   }
@@ -247,68 +256,77 @@ Interface.prototype._insertString = function(c) {
 Interface.prototype._tabComplete = function() {
   var self = this;
 
-  var rv = this.completer(self.line.slice(0, self.cursor));
-  var completions = rv[0],
-      completeOn = rv[1];  // the text that was completed
-  if (completions && completions.length) {
-    // Apply/show completions.
-    if (completions.length === 1) {
-      self._insertString(completions[0].slice(completeOn.length));
-      self._refreshLine();
-    } else {
-      self.output.write('\r\n');
-      var width = completions.reduce(function(a, b) {
-        return a.length > b.length ? a : b;
-      }).length + 2;  // 2 space padding
-      var maxColumns = Math.floor(this.columns / width) || 1;
+  self.pause();
+  self.completer(self.line.slice(0, self.cursor), function(err, rv) {
+    self.resume();
 
-      function handleGroup(group) {
-        if (group.length == 0) {
-          return;
-        }
-        var minRows = Math.ceil(group.length / maxColumns);
-        for (var row = 0; row < minRows; row++) {
-          for (var col = 0; col < maxColumns; col++) {
-            var idx = row * maxColumns + col;
-            if (idx >= group.length) {
-              break;
-            }
-            var item = group[idx];
-            self.output.write(item);
-            if (col < maxColumns - 1) {
-              for (var s = 0, itemLen = item.length; s < width - itemLen; s++) {
-                self.output.write(' ');
+    if (err) {
+      // XXX Log it somewhere?
+      return;
+    }
+
+    var completions = rv[0],
+        completeOn = rv[1];  // the text that was completed
+    if (completions && completions.length) {
+      // Apply/show completions.
+      if (completions.length === 1) {
+        self._insertString(completions[0].slice(completeOn.length));
+      } else {
+        self.output.write('\r\n');
+        var width = completions.reduce(function(a, b) {
+          return a.length > b.length ? a : b;
+        }).length + 2;  // 2 space padding
+        var maxColumns = Math.floor(self.columns / width) || 1;
+
+        function handleGroup(group) {
+          if (group.length == 0) {
+            return;
+          }
+          var minRows = Math.ceil(group.length / maxColumns);
+          for (var row = 0; row < minRows; row++) {
+            for (var col = 0; col < maxColumns; col++) {
+              var idx = row * maxColumns + col;
+              if (idx >= group.length) {
+                break;
+              }
+              var item = group[idx];
+              self.output.write(item);
+              if (col < maxColumns - 1) {
+                for (var s = 0, itemLen = item.length; s < width - itemLen;
+                     s++) {
+                  self.output.write(' ');
+                }
               }
             }
+            self.output.write('\r\n');
           }
           self.output.write('\r\n');
         }
-        self.output.write('\r\n');
-      }
 
-      var group = [], c;
-      for (var i = 0, compLen = completions.length; i < compLen; i++) {
-        c = completions[i];
-        if (c === '') {
-          handleGroup(group);
-          group = [];
-        } else {
-          group.push(c);
+        var group = [], c;
+        for (var i = 0, compLen = completions.length; i < compLen; i++) {
+          c = completions[i];
+          if (c === '') {
+            handleGroup(group);
+            group = [];
+          } else {
+            group.push(c);
+          }
         }
-      }
-      handleGroup(group);
+        handleGroup(group);
 
-      // If there is a common prefix to all matches, then apply that
-      // portion.
-      var f = completions.filter(function(e) { if (e) return e; });
-      var prefix = commonPrefix(f);
-      if (prefix.length > completeOn.length) {
-        self._insertString(prefix.slice(completeOn.length));
-      }
+        // If there is a common prefix to all matches, then apply that
+        // portion.
+        var f = completions.filter(function(e) { if (e) return e; });
+        var prefix = commonPrefix(f);
+        if (prefix.length > completeOn.length) {
+          self._insertString(prefix.slice(completeOn.length));
+        }
 
+      }
       self._refreshLine();
     }
-  }
+  });
 };
 
 
@@ -335,7 +353,7 @@ Interface.prototype._wordLeft = function() {
     this.cursor -= match[0].length;
     this._refreshLine();
   }
-}
+};
 
 
 Interface.prototype._wordRight = function() {
@@ -345,7 +363,7 @@ Interface.prototype._wordRight = function() {
     this.cursor += match[0].length;
     this._refreshLine();
   }
-}
+};
 
 
 Interface.prototype._deleteLeft = function() {
@@ -375,7 +393,7 @@ Interface.prototype._deleteWordLeft = function() {
     this.cursor = leading.length;
     this._refreshLine();
   }
-}
+};
 
 
 Interface.prototype._deleteWordRight = function() {
@@ -386,20 +404,20 @@ Interface.prototype._deleteWordRight = function() {
                 trailing.slice(match[0].length);
     this._refreshLine();
   }
-}
+};
 
 
 Interface.prototype._deleteLineLeft = function() {
   this.line = this.line.slice(this.cursor);
   this.cursor = 0;
   this._refreshLine();
-}
+};
 
 
 Interface.prototype._deleteLineRight = function() {
   this.line = this.line.slice(0, this.cursor);
   this._refreshLine();
-}
+};
 
 
 Interface.prototype._line = function() {
@@ -454,11 +472,11 @@ Interface.prototype._ttyWrite = function(s, key) {
   if (key.ctrl && key.shift) {
     /* Control and shift pressed */
     switch (key.name) {
-      case "backspace":
+      case 'backspace':
         this._deleteLineLeft();
         break;
 
-      case "delete":
+      case 'delete':
         this._deleteLineRight();
         break;
     }
@@ -594,9 +612,7 @@ Interface.prototype._ttyWrite = function(s, key) {
         break;
 
       case 'tab': // tab completion
-        if (this.completer) {
-          this._tabComplete();
-        }
+        this._tabComplete();
         break;
 
       case 'left':
