@@ -912,6 +912,7 @@ function Server(/* [options], listener */) {
   this.setOptions(options);
 
   var sharedCreds = crypto.createCredentials({
+    pfx : self.pfx,
     key: self.key,
     passphrase: self.passphrase,
     cert: self.cert,
@@ -996,6 +997,7 @@ Server.prototype.setOptions = function(options) {
     this.rejectUnauthorized = false;
   }
 
+  if (options.pfx) this.pfx = options.pfx;
   if (options.key) this.key = options.key;
   if (options.passphrase) this.passphrase = options.passphrase;
   if (options.cert) this.cert = options.cert;
