@@ -255,9 +255,7 @@ static void Spin(uv_idle_t* handle, int status) {
   Tick();
 }
 
-
-static Handle<Value> NeedTickCallback(const Arguments& args) {
-  HandleScope scope;
+static void StartTickSpinner() {
   need_tick_cb = true;
   // TODO: this tick_spinner shouldn't be necessary. An ev_prepare should be
   // sufficent, the problem is only in the case of the very last "tick" -
@@ -268,9 +266,12 @@ static Handle<Value> NeedTickCallback(const Arguments& args) {
     uv_idle_start(&tick_spinner, Spin);
     uv_ref(uv_default_loop());
   }
-  return Undefined();
 }
 
+static Handle<Value> NeedTickCallback(const Arguments& args) {
+  StartTickSpinner();
+  return Undefined();
+}
 
 static void PrepareTick(uv_prepare_t* handle, int status) {
   assert(handle == &prepare_tick_watcher);
@@ -1694,6 +1695,9 @@ void FatalException(TryCatch &try_catch) {
   emit->Call(process, 2, event_argv);
   // Decrement so we know if the next exception is a recursion or not
   uncaught_exception_counter--;
+
+  // This makes sure uncaught exceptions don't interfere with process.nextTick
+  StartTickSpinner();
 }
 
 
@@ -1880,12 +1884,9 @@ static Handle<Boolean> EnvDeleter(Local<String> property,
   HandleScope scope;
 #ifdef __POSIX__
   String::Utf8Value key(property);
-  // prototyped as `void unsetenv(const char*)` on some platforms
-  if (unsetenv(*key) < 0) {
-    // Deletion failed. Return true if the key wasn't there in the first place,
-    // false if it is still there.
-    return scope.Close(Boolean::New(getenv(*key) == NULL));
-  };
+  if (!getenv(*key)) return False();
+  unsetenv(*key); // can't check return value, it's void on some platforms
+  return True();
 #else
   String::Value key(property);
   WCHAR* key_ptr = reinterpret_cast<WCHAR*>(*key);
@@ -1896,9 +1897,8 @@ static Handle<Boolean> EnvDeleter(Local<String> property,
               GetLastError() != ERROR_SUCCESS;
     return scope.Close(Boolean::New(rv));
   }
+  return True();
 #endif
-  // It worked
-  return v8::True();
 }
 
 
