@@ -191,7 +191,7 @@ struct StringPtr {
   void Update(const char* str, size_t size) {
     if (str_ == NULL)
       str_ = str;
-    else if (on_heap_ || str_ + size != str) {
+    else if (on_heap_ || str_ + size_ != str) {
       // Non-consecutive input, make a copy on the heap.
       // TODO Use slab allocation, O(n) allocs is bad.
       char* s = new char[size_ + size];
