@@ -33,7 +33,6 @@
 # include <io.h>
 
 # include <platform_win32.h>
-# include <platform_win32_winsock.h>
 #endif
 
 #ifdef __POSIX__
@@ -118,7 +117,7 @@ static Handle<Value> GetCPUInfo(const Arguments& args) {
 
 static Handle<Value> GetFreeMemory(const Arguments& args) {
   HandleScope scope;
-  double amount = Platform::GetFreeMemory();
+  double amount = uv_get_free_memory();
 
   if (amount < 0) {
     return Undefined();
@@ -129,7 +128,7 @@ static Handle<Value> GetFreeMemory(const Arguments& args) {
 
 static Handle<Value> GetTotalMemory(const Arguments& args) {
   HandleScope scope;
-  double amount = Platform::GetTotalMemory();
+  double amount = uv_get_total_memory();
 
   if (amount < 0) {
     return Undefined();
@@ -151,29 +150,22 @@ static Handle<Value> GetUptime(const Arguments& args) {
 
 static Handle<Value> GetLoadAvg(const Arguments& args) {
   HandleScope scope;
-  Local<Array> loads = Array::New(3);
-  int r = Platform::GetLoadAvg(&loads);
+  double loadavg[3];
+  uv_loadavg(loadavg);
 
-  if (r < 0) {
-    return Undefined();
-  }
+  Local<Array> loads = Array::New(3);
+  loads->Set(0, Number::New(loadavg[0]));
+  loads->Set(1, Number::New(loadavg[1]));
+  loads->Set(2, Number::New(loadavg[2]));
 
   return scope.Close(loads);
 }
 
-#ifdef __MINGW32__
-static Handle<Value> OpenOSHandle(const Arguments& args) {
-  HandleScope scope;
 
-  intptr_t handle = args[0]->IntegerValue();
-
-  int fd = _open_osfhandle(handle, 0);
-  if (fd < 0)
-    return ThrowException(ErrnoException(errno, "_open_osfhandle"));
-
-  return scope.Close(Integer::New(fd));
+static Handle<Value> GetInterfaceAddresses(const Arguments& args) {
+  return Platform::GetInterfaceAddresses();
 }
-#endif // __MINGW32__
+
 
 void OS::Initialize(v8::Handle<v8::Object> target) {
   HandleScope scope;
@@ -186,13 +178,10 @@ void OS::Initialize(v8::Handle<v8::Object> target) {
   NODE_SET_METHOD(target, "getCPUs", GetCPUInfo);
   NODE_SET_METHOD(target, "getOSType", GetOSType);
   NODE_SET_METHOD(target, "getOSRelease", GetOSRelease);
-
-#ifdef __MINGW32__
-  NODE_SET_METHOD(target, "openOSHandle", OpenOSHandle);
-#endif
+  NODE_SET_METHOD(target, "getInterfaceAddresses", GetInterfaceAddresses);
 }
 
 
 }  // namespace node
 
-NODE_MODULE(node_os, node::OS::Initialize);
+NODE_MODULE(node_os, node::OS::Initialize)
