@@ -32,6 +32,17 @@
 #include <inttypes.h>
 #include <sys/types.h>
 #include <sys/loadavg.h>
+#include <sys/socket.h>
+#include <net/if.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <fcntl.h>
+
+#ifdef SUNOS_HAVE_IFADDRS
+# include <ifaddrs.h>
+#endif
+
+
 
 #if (!defined(_LP64)) && (_FILE_OFFSET_BITS - 0 == 64)
 #define PROCFS_FILE_OFFSET_BITS_HACK 1
@@ -51,6 +62,7 @@ namespace node {
 
 using namespace v8;
 
+double Platform::prog_start_time = Platform::GetUptime();
 
 char** Platform::SetupArgs(int argc, char *argv[]) {
   return argv;
@@ -68,48 +80,21 @@ const char* Platform::GetProcessTitle(int *len) {
 }
 
 
-int Platform::GetMemory(size_t *rss, size_t *vsize) {
-  pid_t pid = getpid();
-
-  size_t page_size = getpagesize();
-  char pidpath[1024];
-  sprintf(pidpath, "/proc/%d/psinfo", pid);
-
+int Platform::GetMemory(size_t *rss) {
   psinfo_t psinfo;
-  FILE *f = fopen(pidpath, "r");
-  if (!f) return -1;
+  int fd;
 
-  if (fread(&psinfo, sizeof(psinfo_t), 1, f) != 1) {
-    fclose (f);
+  if ((fd = open("/proc/self/psinfo", O_RDONLY)) < 0)
+    return -1;
+
+  if (read(fd, &psinfo, sizeof (psinfo_t)) != sizeof (psinfo_t)) {
+    (void) close(fd);
     return -1;
   }
 
-  /* XXX correct? */
-
-  *vsize = (size_t) psinfo.pr_size * page_size;
   *rss = (size_t) psinfo.pr_rssize * 1024;
+  (void) close(fd);
 
-  fclose (f);
-
-  return 0;
-}
-
-
-int Platform::GetExecutablePath(char* buffer, size_t* size) {
-  const char *execname = getexecname();
-  if (!execname) return -1;
-  if (execname[0] == '/') {
-    char *result = strncpy(buffer, execname, *size);
-    *size = strlen(result);
-  } else {
-    char *result = getcwd(buffer, *size);
-    if (!result) return -1;
-    result = strncat(buffer, "/", *size);
-    if (!result) return -1;
-    result = strncat(buffer, execname, *size);
-    if (!result) return -1;
-    *size = strlen(result);
-  }
   return 0;
 }
 
@@ -137,7 +122,7 @@ static Handle<Value> data_named(kstat_named_t *knp) {
     val = String::New(KSTAT_NAMED_STR_PTR(knp));
     break;
   default:
-    throw (String::New("unrecognized data type"));
+    val = String::New("unrecognized data type");
   }
 
   return (val);
@@ -155,7 +140,7 @@ int Platform::GetCPUInfo(Local<Array> *cpus) {
   kstat_named_t *knp;
 
   if ((kc = kstat_open()) == NULL)
-    throw "could not open kstat";
+    return -1;
 
   *cpus = Array::New();
 
@@ -215,75 +200,99 @@ int Platform::GetCPUInfo(Local<Array> *cpus) {
 }
 
 
-double Platform::GetFreeMemory() {
-  kstat_ctl_t   *kc;
-  kstat_t       *ksp;
-  kstat_named_t *knp;
-
-  double pagesize = static_cast<double>(sysconf(_SC_PAGESIZE));
-  ulong_t freemem;
-
-  if((kc = kstat_open()) == NULL)
-    throw "could not open kstat";
-
-  ksp = kstat_lookup(kc, (char *)"unix", 0, (char *)"system_pages");
-
-  if(kstat_read(kc, ksp, NULL) == -1){
-    throw "could not read kstat";
-  }
-  else {
-    knp = (kstat_named_t *) kstat_data_lookup(ksp, (char *)"freemem");
-    freemem = knp->value.ul;
-  }
-
-  kstat_close(kc);
-
-  return static_cast<double>(freemem)*pagesize;
-}
-
-
-double Platform::GetTotalMemory() {
-  double pagesize = static_cast<double>(sysconf(_SC_PAGESIZE));
-  double pages = static_cast<double>(sysconf(_SC_PHYS_PAGES));
-
-  return pagesize*pages;
-}
-
-double Platform::GetUptime() {
+double Platform::GetUptimeImpl() {
   kstat_ctl_t   *kc;
   kstat_t       *ksp;
   kstat_named_t *knp;
 
   long hz = sysconf(_SC_CLK_TCK);
-  ulong_t clk_intr;
+  double clk_intr;
 
   if ((kc = kstat_open()) == NULL)
-    throw "could not open kstat";
+    return -1;
 
   ksp = kstat_lookup(kc, (char *)"unix", 0, (char *)"system_misc");
 
   if (kstat_read(kc, ksp, NULL) == -1) {
-    throw "unable to read kstat";
+    clk_intr = -1;
   } else {
     knp = (kstat_named_t *) kstat_data_lookup(ksp, (char *)"clk_intr");
-    clk_intr = knp->value.ul;
+    clk_intr = knp->value.ul / hz;
   }
 
   kstat_close(kc);
 
-  return static_cast<double>( clk_intr / hz );
+  return clk_intr;
 }
 
-int Platform::GetLoadAvg(Local<Array> *loads) {
+
+Handle<Value> Platform::GetInterfaceAddresses() {
   HandleScope scope;
-  double loadavg[3];
 
-  (void) getloadavg(loadavg, 3);
-  (*loads)->Set(0, Number::New(loadavg[LOADAVG_1MIN]));
-  (*loads)->Set(1, Number::New(loadavg[LOADAVG_5MIN]));
-  (*loads)->Set(2, Number::New(loadavg[LOADAVG_15MIN]));
+#ifndef SUNOS_HAVE_IFADDRS
+  return ThrowException(Exception::Error(String::New(
+    "This version of sunos doesn't support getifaddrs")));
+#else
+  struct ::ifaddrs *addrs, *ent;
+  struct ::sockaddr_in *in4;
+  struct ::sockaddr_in6 *in6;
+  char ip[INET6_ADDRSTRLEN];
+  Local<Object> ret, o;
+  Local<String> name, ipaddr, family;
+  Local<Array> ifarr;
 
-  return 0;
+  if (getifaddrs(&addrs) != 0) {
+    return ThrowException(ErrnoException(errno, "getifaddrs"));
+  }
+
+  ret = Object::New();
+
+  for (ent = addrs; ent != NULL; ent = ent->ifa_next) {
+    bzero(&ip, sizeof (ip));
+    if (!(ent->ifa_flags & IFF_UP && ent->ifa_flags & IFF_RUNNING)) {
+      continue;
+    }
+
+    if (ent->ifa_addr == NULL) {
+      continue;
+    }
+
+    name = String::New(ent->ifa_name);
+    if (ret->Has(name)) {
+      ifarr = Local<Array>::Cast(ret->Get(name));
+    } else {
+      ifarr = Array::New();
+      ret->Set(name, ifarr);
+    }
+
+    if (ent->ifa_addr->sa_family == AF_INET6) {
+      in6 = (struct sockaddr_in6 *)ent->ifa_addr;
+      inet_ntop(AF_INET6, &(in6->sin6_addr), ip, INET6_ADDRSTRLEN);
+      family = String::New("IPv6");
+    } else if (ent->ifa_addr->sa_family == AF_INET) {
+      in4 = (struct sockaddr_in *)ent->ifa_addr;
+      inet_ntop(AF_INET, &(in4->sin_addr), ip, INET6_ADDRSTRLEN);
+      family = String::New("IPv4");
+    } else {
+      (void) strlcpy(ip, "<unknown sa family>", INET6_ADDRSTRLEN);
+      family = String::New("<unknown>");
+    }
+
+    o = Object::New();
+    o->Set(String::New("address"), String::New(ip));
+    o->Set(String::New("family"), family);
+    o->Set(String::New("internal"), ent->ifa_flags & IFF_PRIVATE || ent->ifa_flags &
+	IFF_LOOPBACK ? True() : False());
+
+    ifarr->Set(ifarr->Length(), o);
+
+  }
+
+  freeifaddrs(addrs);
+
+  return scope.Close(ret);
+
+#endif  // SUNOS_HAVE_IFADDRS
 }
 
 
